//-----------------------------------------------
// CMU800MIDI for LUFA
//    by morecat_lab
//
// based on CMU-800 MIDIKIT AVR Version by RJB
//
// Device   : AT90USB162
// X'tal    : 16MHz
// FUSEݒ : 
// Version  :  1.0     Jan. 11 2008 
//             2.2     Aug. 10 2011  (base version by RJB)
//             0.5     Apr. 23, 2012 : LUFA version
//-----------------------------------------------
// include

#include "lufa.h"

//-----------------------------------------------
// define

#define	A8253CTL1	0x03
#define	A8253CTL2	0x07
#define	A8255PA		0x08
#define	A8255PB		0x09
#define	A8255PC		0x0A
#define	A8255CTL	0x0B

// define for use TEMPO dial to send USB-MIDI sync (experimental)
#undef SEND_SYNC
// define if you want to follow the power state of CMU-800
#define CHECK_POWER

// PIN assign
#define	WR		PD7
#define WR_DISABLE	(PORTD |= (1 << WR))
#define WR_ENABLE	(PORTD &= ~(1 << WR))

#define SYNC_CLK	PD5
#define SYNC_START	PD6
#define SYNC_CLK_HIGH	(PORTD |= (1 << SYNC_CLK))
#define SYNC_CLK_LOW	(PORTD &= ~(1 << SYNC_CLK))
#define SYNC_RUN	(PORTD |= (1 << SYNC_START))
#define SYNC_STOP	(PORTD &= ~(1 << SYNC_START))

#if defined(__AVR_ATmega32U4__)
#define LED_MIDI	PE6
#define LED_MIDI_OFF	(PORTE &= ~(1 << LED_MIDI))
#define LED_MIDI_ON	(PORTE |= (1 << LED_MIDI))
#define LED_BEAT	PD4
#define LED_BEAT_OFF	(PORTD &= ~(1 << LED_BEAT))
#define LED_BEAT_ON	(PORTD |= (1 << LED_BEAT))
#endif

#if defined(__AVR_AT90USB162__)
#define LED_MIDI	PC2
#define LED_MIDI_OFF	(PORTC &= ~(1 << LED_MIDI))
#define LED_MIDI_ON	(PORTC |= (1 << LED_MIDI))
#define LED_BEAT	PC4
#define LED_BEAT_OFF	(PORTC &= ~(1 << LED_BEAT))
#define LED_BEAT_ON	(PORTC |= (1 << LED_BEAT))
#ifdef CHECK_POWER
#define POWER_CHECK	PC5
#define POWER_CHECK_OK 	(bit_is_set(PINC, POWER_CHECK))
#define LED_POWER	PC6
#define LED_POWER_OFF	(PORTC &= ~(1 << LED_POWER))
#define LED_POWER_ON	(PORTC |= (1 << LED_POWER))
#endif
#ifdef SEND_SYNC
#define SYNC_IN		PC7
#define SYNC_PORT	PINC
#endif
#endif


#define ON		1
#define OFF		0
#define INIT		2
#define RHYTHMOFF	0xFF

#define MAX_CH		16
#define MAX_NOTE_CNT	8
#define	INTRCOUNT	(0x10000 - (F_CPU / 64 / 1000))	// ^C} 1ms
#define	BUFSIZE		16		// Oobt@TCY

#define MAX_POLY_VOICE	4
#define OFFSET_CHORD_CH	2

// MIDIbZ[W
#define MIDI_NoteOff			0x80
#define MIDI_NoteOn			0x90
#define MIDI_PolykeyPressure		0xA0
#define MIDI_ControlChange		0xB0
#define MIDI_ProgramChange		0xC0
#define MIDI_ChannelPressure		0xD0
#define MIDI_PitchBend			0xE0

#define MIDI_StartSysEx			0xF0
#define MIDI_TuneRequest		0xF6
#define MIDI_EndSysEx			0xF7

#define MIDI_TimingClock		0xF8
#define MIDI_Start			0xFA
#define MIDI_Continue			0xFB
#define MIDI_Stop			0xFC
#define MIDI_ActiveSensing		0xFE
#define MIDI_SystemReset		0xFF

#define MIDI_CC_Moduration		0x01
#define MIDI_CC_DataEntry		0x06
#define MIDI_CC_Control1		0x10 // MOD SHAPE
#define MIDI_CC_Control2		0x11 // MOD RATE
#define MIDI_CC_Control3		0x12 // MOD DEPTH
#define MIDI_CC_RPN_LSB			0x64
#define MIDI_CC_RPN_MSB			0x65

#define MIDI_MM_AllSoundOff		0x78
#define MIDI_MM_ResetAllControl		0x79
#define MIDI_MM_AllNoteOff		0x7B

/** LUFA MIDI Class driver interface configuration and state information. 
    This structure is passed to all MIDI Class driver functions, so
    that multiple instances of the same class within a device can be 
    differentiated from one another.
**/
USB_ClassInfo_MIDI_Device_t Keyboard_MIDI_Interface = {
  .Config = {
    .StreamingInterfaceNumber = 1,
    
    .DataINEndpointNumber      = MIDI_STREAM_IN_EPNUM,
    .DataINEndpointSize        = MIDI_STREAM_EPSIZE,
    .DataINEndpointDoubleBank  = false,

    .DataOUTEndpointNumber     = MIDI_STREAM_OUT_EPNUM,
    .DataOUTEndpointSize       = MIDI_STREAM_EPSIZE,
    .DataOUTEndpointDoubleBank = false,
  },
};

//-----------------------------------------------
// O[oϐ

uchar	RxByte;
uchar	PC;
uchar	SysEx;
uchar	Rch;
uchar	MByte[2];
uchar	CkCnt;
uchar	CkTim;
uint16	CkPhase;
BOOL	Setflg;

#ifdef SEND_SYNC
#define HW_CDC_BULK_OUT_SIZE 8
uchar  utx_buf[HW_CDC_BULK_OUT_SIZE];
#endif

uchar	CVBuf[MAX_CH];
uchar	PBBuf[MAX_CH];
uchar	MWBuf[MAX_CH];
uchar	RateBuf[MAX_CH];
uchar	ShapeBuf[MAX_CH];
uchar	DepthBuf[MAX_CH];
uchar	RPNMsbBuf[MAX_CH];
uchar	RPNLSBBuf[MAX_CH];
uchar	DetuneBuf[MAX_CH];
uchar	NoteCnt[MAX_CH];
uchar	NoteBuf[MAX_CH][MAX_NOTE_CNT];
uchar	PlayNoteBuf[MAX_CH];

// R[h`lp(CH 9)
uchar	PolyNoteBuf[4];
uchar	PolyNoteOrder[4];
uchar	PolyNoteOrderCnt;

#ifdef CHECK_POWER
uchar powerState = OFF;
uint16 powerStateCounter = 0;
#endif
#ifdef SEND_SYNC
uchar sync_timing = FALSE;
#endif

// ݒpe[u
// Master Oscillator = 2539732Hz  A=440Hz
const uint16	DivBase[12] = {38830, 36650, 34593, 32652, 30819, 29089, 27456, 25915, 24461, 23088, 21792, 20569};
const uchar	DetuneBase[12] = {35, 33, 32, 30, 28, 26, 25, 24, 22, 21, 20, 19};
const uchar 	BendBase[13] = {148, 139, 131, 124, 117, 110, 104, 98, 93, 88, 83, 78, 74};
const uchar 	ModBase[12] = {9, 8, 8, 7, 7, 6, 6, 6,	5, 5, 5, 5};

#ifdef SEND_SYNC
SIGNAL(INT4_vect) {
  sync_timing = TRUE;
}

#endif

//-----------------------------------------------
// ^C}1I[o[t[荞
SIGNAL(TIMER1_OVF_vect) {
  TCNT1 = INTRCOUNT;
  
  CkPhase ++;
  Setflg = TRUE;
  
  if(CkTim++ > 4) SYNC_CLK_LOW;

#ifdef CHECK_POWER
  if (powerState == OFF) {
    if (POWER_CHECK_OK) {
      if (powerStateCounter++ > 1000) { /* wait 1sec for initialize */
	initCMU800();  // CMU-800
	powerState = INIT;
      }
    } else {
      powerStateCounter = 0;
    }
  } else if (powerState == INIT) {
    if (powerStateCounter++ > 3000) { /* wait 2sec before initialize finish */
      powerState = ON;
      LED_POWER_ON;
    }
  } else {			/* powerState == ON */
    if (! POWER_CHECK_OK) {
      powerState = OFF;
      powerStateCounter = 0;
      LED_POWER_OFF;
    }
  }
#endif

}
//-----------------------------------------------
// EGCg

void Wait(int32 i) {
  while(i-- > 0);
}

//-----------------------------------------------
// CMU-800֏o

void OutPort(uchar addr, uchar data) {

  PORTD = (PORTD & 0xF0) | ( addr & 0x0F); // address
  PORTB = data; // data

  WR_ENABLE;
  WR_DISABLE;
}

//-----------------------------------------------
// 8255

void Init8255(void) {
  OutPort(A8255CTL, 0x88);
  OutPort(A8255PA, 0x80);
  OutPort(A8255PB, 0xFF);
  OutPort(A8255PC, 0xFF);
}

//-----------------------------------------------
// 8253

void Init8253(void) {
  // 8253 #1 CH1-CH3
  OutPort(A8253CTL1, 0x36);
  OutPort(A8253CTL1, 0x76);
  OutPort(A8253CTL1, 0xB6);

  // 8253 #2 CH4-CH6
  OutPort(A8253CTL2, 0x36);
  OutPort(A8253CTL2, 0x76);
  OutPort(A8253CTL2, 0xB6);
}

//-----------------------------------------------
// GATEo

void Gate(uchar ch, uchar onoff) {
  if(ch > 0x07)
    return;

  OutPort(A8255PC, (ch << 1) + 1);

  if(onoff) {
    CVBuf[ch] &= ~0x80;
  } else {
    CVBuf[ch] |= 0x80;
  }
  OutPort(A8255PA, CVBuf[ch]);

  OutPort(A8255PC, ch << 1);
  Wait(20);	// Wait 50us
  OutPort(A8255PC, (ch << 1) + 1);
}

//-----------------------------------------------
// SGATEo͂OFF

void AllGateOff(void) {
  uchar ch;

  for(ch = 0; ch < 8; ch++) {
    OutPort(A8255PC, (ch << 1) + 1);
    OutPort(A8255PA, 0x80);
    
    OutPort(A8255PC, ch << 1);
    Wait(60);	// Wait 50us
    OutPort(A8255PC, (ch << 1) + 1);
  }
}

//-----------------------------------------------
// CVo͓dZbg

void SetCV(uchar ch) {
  uchar note;

  note = PlayNoteBuf[ch];
  if( note < 24) {
    note = 24;
  }
  note -= 24;

  while(note > 63) {
    note -= 12;
  }

  OutPort(A8255PC, (ch << 1) + 1);
  
  CVBuf[ch] = (CVBuf[ch] & 0x80) | (note & 0x3F);
  OutPort(A8255PA, CVBuf[ch]);

  OutPort(A8255PC, ch << 1);
  Wait(20);	// Wait 50us
  OutPort(A8255PC, (ch << 1) + 1);
}

//-----------------------------------------------
// DCOgZbg

void SetDCO(uchar ch) {
  uchar	note,ctn,val,ph;
  schar	lfo = 0;
  uint16 m,n,div,bend;
  int16	mod,detune;

  if(ch > 0x05)
    return;
  note = PlayNoteBuf[ch];

  ctn = ch; 
  if( ctn > 2)
    ctn++;

  // {
  if(note < 24)
    note = 24;
  note -= 24;
  m = note / 12;
  n = note % 12;
  div = DivBase[n] >> m;

  // f`[ʂɂ镪ω
  detune = (DetuneBuf[ch] - 64) * DetuneBase[n];

  // W[Vg
  if(RateBuf[ch] < 3) {
    ph = CkPhase >> (3 - RateBuf[ch]);
  } else {
    ph = CkPhase << (RateBuf[ch] - 3); 
  }

  // W[Vg`
  switch (ShapeBuf[ch]) {
  case 0:	// Opg
    if (ph > 128) {
      lfo = 191 - ph;
    } else {
      lfo = ph - 64;
    }
    break;
  case 1: // mRMg
    lfo = 63 - (ph >> 1);
    break;
  case 2: // tmRMg
    lfo = (ph >> 1) - 64;
    break;
  case 3: // `g	
    if(ph > 128) {
      lfo = -64;
    } else {
      lfo = 63;
    }
    break;
  }
  // W[Vʂɂ镪ω
  mod = (MWBuf[ch] * ModBase[n] * lfo) >> (4 - DepthBuf[ch]);
  
  // sb`xhʂŏI
  val = PBBuf[ch];
  if(val < 64) {
    bend = (BendBase[n] * (64 - val)) >> 1;
    div = (DivBase[n] - detune + bend + mod) >> m;
  } else {
    bend = (BendBase[n+1] * (val - 64)) >> 1;
    div = (DivBase[n] - detune - bend + mod) >> m;
  }
  
  // 8253ɃZbg
  OutPort(ctn, (div & 0xFF));	// LSB
  OutPort(ctn, (div >> 8));	// MSB
} 

//-----------------------------------------------
// YgK[

void RhythmTrig(uchar note) {
  uchar d;

  switch(note){
  case 35: d = 0x80; break; // ABD
  case 36: d = 0x80; break; // BD
  case 40: d = 0x40; break; // ESD
  case 38: d = 0x40; break; // SD
  case 41: d = 0x20; break; // LFT
  case 43: d = 0x20; break; // HFT
  case 45: d = 0x20; break; // LT
  case 47: d = 0x20; break; // LMT
  case 48: d = 0x10; break; // HMT
  case 50: d = 0x10; break; // HT
  case 49: d = 0x08; break; // CR1
  case 57: d = 0x08; break; // CR2
  case 46: d = 0x04; break; // OH
  case 42: d = 0x02; break; // CH
  default: d = 0x00; break;
  }
  OutPort(A8255PB, ~d);
}

//-----------------------------------------------
// m[gON(poly)
void PolyNoteON(void) {
  uchar i;
  uchar min = 0xff;

  // MIDI9chȊO͖
  if(Rch != 0x08)
    return;

  // Find oldest Note
  for(i = 0; i < MAX_POLY_VOICE; i++) {
    if(PolyNoteOrder[i] < min) {
      min = PolyNoteOrder[i];
    }
  }

  for(i = 0; i < MAX_POLY_VOICE; i++) {
    if(PolyNoteOrder[i] == min)
      break;
  }

  if(min != 0) {
    Gate(i+2,OFF);	// ReTrigger
  }

  PolyNoteBuf[i] = MByte[0];
  PolyNoteOrder[i] = PolyNoteOrderCnt;
  PolyNoteOrderCnt++;

  PlayNoteBuf[i + OFFSET_CHORD_CH] = PolyNoteBuf[i];
  SetCV(i + OFFSET_CHORD_CH);
  SetDCO(i + OFFSET_CHORD_CH);
  Gate(i + OFFSET_CHORD_CH,ON);
}

//-----------------------------------------------
// m[gOFF(Poly)

void PolyNoteOFF(void) {
  uchar i;
  uchar flg = TRUE;

  // MIDI9chȊO͖
  if(Rch != 0x08)
    return;

  // obt@ɓo^Ăꍇ͍폜
  for(i = 0; i < MAX_POLY_VOICE; i++) {
    if(PolyNoteBuf[i] == MByte[0]) {
      Gate(i + OFFSET_CHORD_CH,OFF);	// GATE OFF
      PolyNoteOrder[i] = 0;
    }
    if( PolyNoteOrder[i]) {
      flg = FALSE;
    }
  }

  // PolyNoteOrderS0 
  if(flg)  {
    PolyNoteOrderCnt = 1;
  }
}

//-----------------------------------------------
// m[gON

void NoteON(void) {
  uchar i;
  uchar max = 0;

  // MIDI9chȏ͖
  if(Rch > 0x07)
    return;

  // obt@ɓo^ς̃m[g͖
  for(i = 0; i < NoteCnt[Rch]; i++) {
    if(NoteBuf[Rch][i] == MByte[0]) {
      return;
    }
  }

  // obt@ςH
  if(NoteCnt[Rch] == MAX_NOTE_CNT) {
    // ʓ˂
    for(i = 0; i < (MAX_NOTE_CNT - 1); i++) {
      NoteBuf[Rch][i] = NoteBuf[Rch][i+1];
    }
    NoteBuf[Rch][MAX_NOTE_CNT - 1] = MByte[0];
  }
  else {
    NoteBuf[Rch][NoteCnt[Rch]] = MByte[0];
    NoteCnt[Rch]++;
  }

  // ō
  for(i = 0; i < NoteCnt[Rch]; i++) {
    if(max < NoteBuf[Rch][i]) {
      max = NoteBuf[Rch][i];
    }
  }
  PlayNoteBuf[Rch] = max;

  SetCV(Rch);
  SetDCO(Rch);
  Gate(Rch, ON);
}

//-----------------------------------------------
// m[gOFF

void NoteOFF(void) {
  uchar i;
  uchar max = 0;
  BOOL flg = FALSE;

  // MIDI9chȏ͖
  if(Rch > 0x07) return;

  // obt@ɓo^Ăꍇ͍폜
  for(i = 0; i < NoteCnt[Rch]; i++) {
    if(flg) {
      NoteBuf[Rch][i-1] = NoteBuf[Rch][i];
    }
    if(NoteBuf[Rch][i] == MByte[0]) {
      flg = TRUE;
    }
  }
  if(flg) NoteCnt[Rch]--;
  
  if(NoteCnt[Rch] == 0) {
    // obt@J̏ꍇGATEIt
    Gate(Rch, OFF);
  } else {
    // ō
    for(i = 0; i < NoteCnt[Rch]; i++) {
      if(max < NoteBuf[Rch][i]) {
	max = NoteBuf[Rch][i];
      }
    }
    PlayNoteBuf[Rch] = max;
    
    SetCV(Rch);
    SetDCO(Rch);
  }
}

void parseUSBMidiMessage(uchar *data, uchar len) {
  /* note: Please refer page 16, midi10.pdf */
  uchar cin = (*data) & 0x0f;	/* CIN(CODE INDEX NUMBER), CABLE NO𖳎 */
  Rch = (*(data+1)) & 0xf;	/* receive channel */
  MByte[0] = *(data+2);
  MByte[1] = *(data+3);

  LED_MIDI_ON;

  switch (cin) {
  case 0:			/* misc function code (reserved) */
  case 1:			/* cable events (reserved) */
  case 2:			/* two-byte system common (2byte) */
  case 3:			/* three-byte system common (3byte) */
  case 4:			/* sys-ex status continues (3byte) */
  case 5:			/* single byte system common (1byte) */
  case 6:			/* system ex end with 2 bytes (2byte) */
  case 7:			/* system ex end with 3 bytes (3byte) */
    break;

  case 8:			/* note off */
    PolyNoteOFF();
    NoteOFF();
    RhythmTrig(RHYTHMOFF);
    LED_MIDI_OFF;
    break;

  case 9:			/* note on  */
    if(MByte[1] == 0){
      PolyNoteOFF();
      NoteOFF();
      RhythmTrig(RHYTHMOFF);
      LED_MIDI_OFF;
    }
    else{
      PolyNoteON();
      NoteON();
      if(Rch == 0x09) {
	RhythmTrig(MByte[0]);
      }
      LED_MIDI_ON;
    }
    break;

  case 0xa:			/* ploy key press */
    break;

  case 0xb:			/* control change */
    switch(MByte[0]) {
    case MIDI_CC_Moduration:
      if(Rch==8) {
	MWBuf[2] = MByte[1] >> 2;
	MWBuf[3] = MByte[1] >> 2;
	MWBuf[4] = MByte[1] >> 2;
	MWBuf[5] = MByte[1] >> 2;
      } else {
	MWBuf[Rch] = MByte[1] >> 2;
      }
      break;
      break;
    case MIDI_CC_DataEntry:
      if((RPNMsbBuf[Rch] == 0) && (RPNLSBBuf[Rch] == 1))
	DetuneBuf[Rch] = MByte[1];
      break;
    case MIDI_CC_Control1:
      ShapeBuf[Rch] = MByte[1] >> 5;
      break;
    case MIDI_CC_Control2:
      RateBuf[Rch] = MByte[1] >> 4;
      break;
    case MIDI_CC_Control3:
      DepthBuf[Rch] = MByte[1] >> 5;
      break;
    case MIDI_CC_RPN_LSB:
      RPNLSBBuf[Rch] = MByte[1];
      break;
    case MIDI_CC_RPN_MSB:
      RPNMsbBuf[Rch] = MByte[1];
      break;
    case MIDI_MM_AllSoundOff:
      NoteCnt[Rch] = 0;
      Gate(Rch, OFF);
      break;
    case MIDI_MM_ResetAllControl:
      PBBuf[Rch] = 64;
      MWBuf[Rch] = 0;
      RateBuf[Rch] = 4;
      ShapeBuf[Rch] = 0;
      DepthBuf[Rch] = 0;
      RPNMsbBuf[Rch] = 127;
      RPNLSBBuf[Rch] = 127;
      DetuneBuf[Rch] = 64;
      break;
    case MIDI_MM_AllNoteOff:
      NoteCnt[Rch] = 0;
      Gate(Rch, OFF);
      break;
    }
    break;

  case 0xc:			/* program change */
    break;

  case 0xd:			/* channel pressure */
    break;

  case 0xe:			/* pitch bend change */
    if(Rch==8) {
      PBBuf[2] = MByte[1];
      PBBuf[3] = MByte[1];
      PBBuf[4] = MByte[1];
      PBBuf[5] = MByte[1];
    }
    else {
      PBBuf[Rch] = MByte[1];
    }
    break;

  case 0xf:			/* single byte */
    switch(*(data+1)) {		/* realtime message */
    case MIDI_TimingClock:
      SYNC_CLK_HIGH;
      CkTim = 0;
      CkCnt++;
      if(CkCnt > 23) {
	CkCnt = 0;
	LED_BEAT_ON;
      }
      if(CkCnt == 5) {
	LED_BEAT_OFF;
      }
      break;
    case MIDI_Start:
      CkCnt = 0;
      SYNC_RUN;
      break;
    case MIDI_Continue:
      SYNC_RUN;
      break;
    case MIDI_Stop:
      SYNC_STOP;
      break;
    case MIDI_ActiveSensing:
      break;
    case MIDI_SystemReset:
      RhythmTrig(RHYTHMOFF);
      AllGateOff();
      break;
    }

  default:
    break;
  }

  LED_MIDI_OFF;

  if (len > 4) {
    parseUSBMidiMessage(data+4, len-4);
  }
}

//-----------------------------------------------
// C

int16 main(void)
{
  uchar i;
  uchar c = 0;

  SetupHardware();

#ifdef CHECK_POWER
  if (POWER_CHECK_OK) {
    initCMU800();  // CMU-800
    powerState = ON;
    LED_POWER_ON;
  }
  Wait(300000);
#else
    initCMU800();  // CMU-800
#endif

  // obt@ϐ
  for(i = 0; i < MAX_CH; i++)	{
    CVBuf[i] = 0x80;
    PBBuf[i] = 64;
    MWBuf[i] = 0;
    RateBuf[i] = 4;
    ShapeBuf[i] = 0;
    DepthBuf[i] = 0;
    NoteCnt[i] = 0;
    RPNMsbBuf[i] = 127;
    RPNLSBBuf[i] = 127;
    DetuneBuf[i] = 64;
  }

  for(i = 0; i < 4; i++)	{
    PolyNoteBuf[i] = 0;
    PolyNoteOrder[i] = 0;
  }
  PolyNoteOrderCnt = 1;

#ifdef SEND_SYNC
  utx_buf[0] = 0x0f;
  utx_buf[1] = MIDI_TimingClock;
  utx_buf[2] = 0;
  utx_buf[3] = 0;
#endif

  sei();
  for (;;) {
    /* receive from USB MIDI */
    MIDI_EventPacket_t ReceivedMIDIEvent;
    while (MIDI_Device_ReceiveEventPacket(&Keyboard_MIDI_Interface, &ReceivedMIDIEvent)) {
      /* for each MIDI packet w/ 4 bytes */
      parseUSBMidiMessage((uchar *)&ReceivedMIDIEvent, 4);
    }
	  
    if(Setflg) {
      Setflg = FALSE;
      // CVd`[W
      SetCV(c & 0x03);			// 1-4ch
      SetCV((c & 0x03) + 0x04);	// 5-8ch
      c++;
      for(i = 0; i < 6; i++) {
	SetDCO(i);
      }
    }

#ifdef SEND_SYNC
    /* send packets to USB MIDI */
    if( sync_timing ) {
      MIDI_Device_SendEventPacket(&Keyboard_MIDI_Interface, (MIDI_EventPacket_t *)&utx_buf);
      MIDI_Device_Flush(&Keyboard_MIDI_Interface);
      sync_timing = FALSE;
    }
#endif

    MIDI_Device_USBTask(&Keyboard_MIDI_Interface);
    USB_USBTask();
  } /* for */

  return(0);
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware(void) {
  /* Disable watchdog if enabled by bootloader/fuses */
  MCUSR &= ~(1 << WDRF);
  wdt_disable();
  
#if defined(__AVR_ATmega8U2__)
#else
  /* Disable clock division */
  clock_prescale_set(clock_div_1);
#endif

  /* Hardware Initialization */

  DDRB = 0b11111111;
  DDRD = 0b11111111;
#if defined(__AVR_ATmega32U4__)
  DDRE = (1 << LED_MIDI);
#endif
#if defined(__AVR_AT90USB162__)
  DDRC = (1 << LED_MIDI) | (1 << LED_BEAT) 
#ifdef CHECK_POWER
| ( 1 << LED_POWER)
#endif
    ;
#endif

#ifdef SEND_SYNC
  EICRB = (1 << ISC41);		/* falling edge of INT4 */
  EIMSK = (1 << INT4);		/* INT4 enable */
#endif

  WR_DISABLE;
  LED_MIDI_OFF;
  LED_BEAT_OFF;
  SYNC_CLK_LOW;
  SYNC_STOP;
  
#if USE_USART
  /* set baud rate */
  UBRR1L = 31;			/* 312500Hz at 16MHz clock */
  /*  */
  UCSR1B = (1<<RXEN1) | (1<<TXEN1);
#endif

  // ^C}1ݒ
  TCCR1A = 0;			/* NORMAL MODE */
  TCCR1B = 3;			/* CLOCK = CLK/64 */
  TIFR1 |= (1 << TOV1);
  TIMSK1 |= (1<< TOIE1);
  TCNT1 = INTRCOUNT;
  
  USB_Init();
}

void initCMU800 (void) {  // CMU-800
  WR_DISABLE;
  Init8255();
  Init8253();
  AllGateOff();
}

/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void) {
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void) {
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void) {
  bool ConfigSuccess = true;

  ConfigSuccess &= MIDI_Device_ConfigureEndpoints(&Keyboard_MIDI_Interface);
}

/** Event handler for the library USB Control Request reception event. */
void EVENT_USB_Device_ControlRequest(void) {
  MIDI_Device_ProcessControlRequest(&Keyboard_MIDI_Interface);
}

/* EOF */
